/* RECORD and PLAYBACK WITH DELAY.
 * record sound from "line in" into a memory buffer.
 * while recording, play back the old contents of the memory buffer.
 * this causes a delay between recording and playback.
 */

.include "ubc-de1media-macros.s"

.global _start

.text
_start:	movia	r23, IOBASE

restart:	movia	r22, WAV_START
		movia	r21, WAV_END

loop:		/* read OLD sounds sample from memory buffer
		 */
		ldh	r4, 0(r22)		/* LEFT  */
		ldh	r5, 2(r22)		/* RIGHT */
		call	playsound

		/* write NEW sample to memory buffer
		 * this remembers the sound until later,
		 * giving the effect of a time delay
		 */
		call	recsound
		sth	r2, 0(r22)
		sth	r3, 2(r22)

		/* advance one position in memory buffer. after this,
		 * r22 will point to the oldest sample in memory.
		 */
		addi	r22, r22, 4
		bltu	r22, r21, loop
		br	restart

/* ***************************************************************** */

recsound:	ldwio	r2, SNDRDY(r23)
		andi	r2, r2, 0xff	/* r2: # samples ready in buffer */
		beq	r2, r0, recsound	/* if ADC buffer is empty, wait */
record:	ldhio r2, SNDL(r23)	/* get new sound sample LEFT */
		ldhio	r3, SNDR(r23)	/*                      RIGHT */
		ret

/* ***************************************************************** */

playsound:	ldwio	r2, SNDRDY(r23)
		srli	r2, r2, 24		/* r2: # samples avail in buffer */
		beq	r2, r0, playsound	/* if DAC buffer is full, wait */
play:		sthio	r4, SNDL(r23)	/* send sound sample to DAC */
		sthio	r5, SNDR(r23)
		ret

.data

WAV_START:
.skip (512*1024)
WAV_END:

.end