/* ECHO BOX.
 * record sound from microphone and play it on audio-out.
 * while recording, keep a copy in a small memory buffer &
 * mix the current sound with weakened sound from the past.
 */

.include "ubc-de1media-macros.s"


.global _start

.text
_start:	movia	r23, IOBASE

restart:	movia	r22, WAV_START
		movia	r21, WAV_END


loop:		/* read OLD sounds sample from memory buffer
		 */
		ldh	r4, 0(r22)		/* LEFT  */
		ldh	r5, 2(r22)		/* RIGHT */
		srai	r4, r4, 1		/* reduce old sound to 1/2 volume */
		srai	r5, r5, 1

		/* mix: add old sample to new sample, and store
		 * back in memory buffer. we'll read it back to
		 * form another echo later.
		 */
		call	recsound
		slli	r2, r2, 2		/* make new sound louder */
		slli	r3, r3, 2
		add	r4, r4, r2		/* add new sound to old sound */
		add	r5, r5, r3
		sth	r4, 0(r22)		/* save new sound in memory */
		sth	r5, 2(r22)

		call	playsound		/* play new sound sample */

		/* advance one position in memory buffer. after this,
		 * r22 will point to the oldest sample in memory.
		 */
		addi	r22, r22, 4
		bltu	r22, r21, loop

		br	restart


/* ***************************************************************** */

recsound:	ldwio	r2, SNDRDY(r23)
		andi	r2, r2, 0xff	/* r2: # samples ready in buffer */
		beq	r2, r0, recsound	/* if buffer is empty, wait */
record:	ldhio r2, SNDL(r23)	/* get new sound sample LEFT */
		ldhio	r3, SNDR(r23)	/*                      RIGHT */
		ret


/* ***************************************************************** */

playsound:	ldwio	r2, SNDRDY(r23)
		srli	r2, r2, 24		/* r2: # samples avail in buffer */
		beq	r2, r0, playsound	/* if DAC buffer is full, wait */
play:		sthio	r4, SNDL(r23)	/* send sound sample to DAC */
		sthio	r5, SNDR(r23)
		ret


/* Wav data is 16 bit (signed), stereo, sampled at 44.1kHz */

.data

WAV_START:
.skip (48*1024)
WAV_END:

.end
